@@ -47,6 +47,12 @@ See L<http://tedia2sql.tigris.org/usingtedia2sql.html>
 
 =item * Preliminary support for Dia's I<database> shapes is added. 
 
+=item * Table comments are used as I<table postfix options>.  This
+means per-table options (e.g. partitioning options) are supported on a
+per-database level.
+
+=item * I<backticks> notation is supported for MySQL-InnoDB.
+
 =back
 
 =head1 DIA VERSIONS
@@ -75,6 +81,8 @@ The following databases are supported:
 
 =item SQLite3
 
+=item SQLite3fk (with foreign key support)
+
 =item Sybase
 
 =item MySQL InnoDB
@@ -159,7 +167,7 @@ This program is released under the GNU General Public License.
 
 =head1 TERMINOLOGY
 
-By I<database> we mean relational database managment system (RDBMS).
+By I<database> we mean relational database management system (RDBMS).
 
 =cut
 
@@ -179,18 +187,20 @@ use Parse::Dia::SQL::Const;
 use Parse::Dia::SQL::Output;
 
 use Parse::Dia::SQL::Output::DB2;
-use Parse::Dia::SQL::Output::Ingres;
+use Parse::Dia::SQL::Output::HTML;
 use Parse::Dia::SQL::Output::Informix;
+use Parse::Dia::SQL::Output::Ingres;
 use Parse::Dia::SQL::Output::MySQL::InnoDB;
 use Parse::Dia::SQL::Output::MySQL::MyISAM;
 use Parse::Dia::SQL::Output::MySQL;
 use Parse::Dia::SQL::Output::Oracle;
 use Parse::Dia::SQL::Output::Postgres;
+use Parse::Dia::SQL::Output::SQLite3;
+use Parse::Dia::SQL::Output::SQLite3fk;
 use Parse::Dia::SQL::Output::Sas;
 use Parse::Dia::SQL::Output::Sybase;
-use Parse::Dia::SQL::Output::SQLite3;
 
-our $VERSION = '0.17';
+our $VERSION = '0.29';
 
 my $UML_ASSOCIATION  = 'UML - Association';
 my $UML_SMALLPACKAGE = 'UML - SmallPackage';
@@ -242,6 +252,8 @@ sub new {
     ignore_type_mismatch => $param{ignore_type_mismatch} || undef,
     converted            => 0,
     loglevel    => $param{loglevel} || undef,
+    backticks   => $param{backticks} || undef,      # MySQL-InnoDB only
+    htmlformat  => $param{htmlformat} || '',        # HTML output only
   };
 
   bless($self, $class);
@@ -317,7 +329,7 @@ sub get_output_instance {
   # Add some args to param unless they are set by caller
   %param =
     map { $param{$_} = $self->{$_} unless exists($param{$_}); $_ => $param{$_} }
-      qw(classes associations small_packages components files index_options typemap loglevel);
+      qw(classes associations small_packages components files index_options typemap loglevel backticks htmlformat);
 
   if ($self->{db} eq q{db2}) {
   return Parse::Dia::SQL::Output::DB2->new(%param);
@@ -339,6 +351,10 @@ sub get_output_instance {
     return Parse::Dia::SQL::Output::Sas->new(%param);
   } elsif ($self->{db} eq q{sqlite3}) {
     return Parse::Dia::SQL::Output::SQLite3->new(%param);
+  } elsif ($self->{db} eq q{sqlite3fk}) {
+    return Parse::Dia::SQL::Output::SQLite3fk->new(%param);
+  } elsif ($self->{db} eq q{html}) {
+    return Parse::Dia::SQL::Output::HTML->new(%param);
   }
 
   return $self->{log}->logdie(qq{Failed to get instance for } . $self->{db});
@@ -863,14 +879,22 @@ sub _parse_database_table {
       $self->{utils}
       ->get_value_from_object( $singleAttrib, "dia:attribute", "name", "comment",
       "string", 1 );
-    $attribComment =~ s/\n/ /g;
+
+    my $attribNullable =
+      $self->{utils}
+      ->get_value_from_object( $singleAttrib, "dia:attribute", "name", "nullable",
+      "boolean", 1 );
+
+	# Strip newlines from comments except for HTML output
+    $attribComment =~ s/\n/ /g if ($self->{db} ne q{html});
+	chomp($attribComment); # Always strip any trailing newlines
 
     $self->{log}->debug(
     "attribute: $attribName - $attribType - $attribVal - $attrib_is_primary_key"
     );
     my $att = [
       $attribName,       $attribType, $attribVal,
-      $attrib_is_primary_key, $attribComment
+      $attrib_is_primary_key, $attribComment, $attribNullable
     ];
     push @{ $classLookup->{attList} }, $att;
 
@@ -1009,7 +1033,10 @@ sub _parse_class {
       $self->{utils}
       ->get_value_from_object( $singleAttrib, "dia:attribute", "name", "comment",
       "string", 1 );
-    $attribComment =~ s/\n/ /g;
+
+	# Strip newlines from comments except for HTML output
+    $attribComment =~ s/\n/ /g if ($self->{db} ne q{html});
+	chomp($attribComment); # Always strip any trailing newlines
 
     $self->{log}->debug(
     "attribute: $attribName - $attribType - $attribVal - $attribVisibility"
@@ -1235,10 +1262,10 @@ sub _parse_association {
         $leftEnd{role} = $self->{utils}->get_string_from_node($currentNode);
       } elsif ( $nodeAttrName eq 'role_b' ) {
         $rightEnd{role} = $self->{utils}->get_string_from_node($currentNode);
-      } elsif ( $nodeAttrName eq 'aggregate_a' ) {
-        $leftEnd{aggregate} = $self->{utils}->get_string_from_node($currentNode);
-      } elsif ( $nodeAttrName eq 'aggregate_b' ) {
-        $rightEnd{aggregate} = $self->{utils}->get_string_from_node($currentNode);
+      } elsif ( $self->{uml} && $nodeAttrName eq 'assoc_type' && $assocDirection == 2 ) {
+        $leftEnd{aggregate} = $self->{utils}->get_num_from_node($currentNode);
+      } elsif ( $self->{uml} && $nodeAttrName eq 'assoc_type' ) {
+        $rightEnd{aggregate} = $self->{utils}->get_num_from_node($currentNode);
       } elsif ( $nodeAttrName eq 'class_scope_a' ) {
         $leftEnd{class_scope} = $self->{utils}->get_string_from_node($currentNode);
       } elsif ( $nodeAttrName eq 'class_scope_b' ) {
